@@ -21,6 +21,8 @@
 
 #include "ccl_clauses.h"
 #include "ccl_tformulae.h"
+#include "cte_lambda.h"
+#include "ccl_clausesets.h"
 
 
 /*---------------------------------------------------------------------*/
@@ -148,17 +150,16 @@ static void clause_collect_posneg_vars(Clause_p clause,
                                 PTree_p *neg_vars)
 {
    Eqn_p handle;
-   long collected = 0;
 
    for(handle=clause->literals; handle; handle = handle->next)
    {
       if(EqnIsPositive(handle))
       {
-         collected += EqnCollectVariables(handle, pos_vars);
+         EqnCollectVariables(handle, pos_vars);
       }
       else
       {
-         collected += EqnCollectVariables(handle, neg_vars);
+         EqnCollectVariables(handle, neg_vars);
       }
    }
 }
@@ -898,13 +899,50 @@ EqnSide ClauseIsEqDefinition(Clause_p clause, int min_arity)
 {
    assert(clause);
 
-   if(problemType == PROBLEM_FO && ClauseIsUnit(clause))
+   if(ClauseIsUnit(clause))
    {
       return EqnIsDefinition(clause->literals, min_arity);
    }
    return NoSide;
 }
 
+/*-----------------------------------------------------------------------
+//
+// Function: ClauseExtractHODefinition()
+//
+//   Given a
+//
+// Global Variables: -
+//
+// Side Effects    : -
+//
+/----------------------------------------------------------------------*/
+
+void ClauseExtractHODefinition(Clause_p clause, EqnSide def_side, Term_p *lside, Term_p* rside)
+{
+   assert(clause);
+   assert(def_side == LeftSide || def_side == RightSide);
+   PStack_p vars = PStackAlloc();
+   const Eqn_p lit = clause->literals;
+   assert(lside != &(lit->lterm) && rside != &(lit->rterm));
+   Term_p def_term = def_side == LeftSide ? lit->lterm : lit->rterm;
+   for(int i=0; i<def_term->arity; i++)
+   {
+      assert(TermIsFreeVar(def_term->args[i]));
+      PStackPushP(vars, def_term->args[i]);
+   }
+   Term_p other_term = def_side == LeftSide ? lit->rterm : lit->lterm;
+   Term_p abstracted_term = AbstractVars(lit->bank, other_term, vars);
+   Term_p symbol = TermTopAlloc(def_term->f_code, 0);
+   symbol->type = abstracted_term->type;
+   assert(symbol->type == SigGetType(lit->bank->sig, symbol->f_code));
+   symbol = TBTermTopInsert(lit->bank, symbol);
+   PStackFree(vars);
+
+   *lside = symbol;
+   *rside = abstracted_term;
+}
+
 
 /*-----------------------------------------------------------------------
 //
@@ -1271,6 +1309,10 @@ void ClausePrintLOPFormat(FILE* out, Clause_p clause, bool fullterms)
 
 void ClausePrint(FILE* out, Clause_p clause, bool fullterms)
 {
+   if(out == stderr)
+   {
+      fprintf(stderr, "ClausePrint(stderr,...)\n");
+   }
 
    /* fprintf(out, "(%ld, %ld)", clause->proof_depth,
       clause->proof_size); */
@@ -1301,6 +1343,33 @@ void ClausePrint(FILE* out, Clause_p clause, bool fullterms)
    }
 }
 
+/*-----------------------------------------------------------------------
+//
+// Function: ClausePrintDBG()
+//
+//   Print a clause in the form useful for debugging.
+//
+// Global Variables: OutputFormat
+//
+// Side Effects    : Output
+//
+/----------------------------------------------------------------------*/
+
+void ClausePrintDBG(FILE* out, Clause_p clause)
+{
+   fprintf(out, "thf(cl%ld, plain, ",
+           clause->ident >= 0 ? clause->ident : clause->ident-LONG_MIN);
+   if(clause->literals)
+   {
+      EqnPrintDBG(out, clause->literals);
+      for(Eqn_p lit = clause->literals->next; lit; lit = lit->next)
+      {
+         DBG_PRINT(out, " | ", EqnPrintDBG(out, lit), "");
+      }
+   }
+   fprintf(out, " ).");
+}
+
 
 /*-----------------------------------------------------------------------
 //
@@ -1462,6 +1531,7 @@ void ClauseTSTPPrint(FILE* out, Clause_p clause, bool fullterms, bool complete)
       form = TFormulaClosure(clause->literals->bank, form, true);
 
       TFormulaTPTPPrint(out, clause->literals->bank, form, fullterms, false);
+      // TermPrintDbg(out, form, clause->literals->bank->sig, DEREF_NEVER);
       // handled by GC, no need to free
    }
 
@@ -1656,6 +1726,7 @@ Clause_p ClauseParse(Scanner_p in, TB_p bank)
    else
    {
       concl = EqnListParse(in, bank, Semicolon);
+      //printf("Parsed EqnList\n");
       if(TestInpTok(in, Colon))
       {
          if(EqnListLength(concl)>1)
@@ -2456,7 +2527,7 @@ long ClauseReturnFCodes(Clause_p clause, PStack_p f_codes)
    for(i=0; i<PStackGetSP(stack);i++)
    {
       t = PStackElementP(stack,i);
-      if(!TermIsVar(t))
+      if(!TermIsAnyVar(t))
       {
          if(!SigQueryFuncProp(sig, t->f_code, FPOpFlag))
          {
@@ -2530,6 +2601,71 @@ bool ClauseQueryLiteral(Clause_p clause, bool (*query_fun)(Eqn_p))
    return false;
 }
 
+/*-----------------------------------------------------------------------
+//
+// Function: ClauseRecognizeChoice()
+//
+//   If the clause is of the form ~P X | P (f P) it will recognize
+//   that f is a defined choice operatior, store f in choice_symbols
+//   map and return true.
+//
+// Global Variables: -
+//
+// Side Effects    : -
+//
+/----------------------------------------------------------------------*/
+
+#define FAIL_ON(cond) if (cond) return false
+
+bool ClauseRecognizeChoice(IntMap_p choice_symbols_map, Clause_p cl)
+{
+   FAIL_ON(cl->pos_lit_no != 1);
+   FAIL_ON(cl->neg_lit_no != 1);
+
+   Eqn_p pos_lit =
+      EqnIsPositive(cl->literals) ? cl->literals : cl->literals->next;
+   Eqn_p neg_lit =
+      EqnIsNegative(cl->literals) ? cl->literals : cl->literals->next;
+   assert(EqnIsPositive(pos_lit));
+   assert(EqnIsNegative(neg_lit));
+
+   FAIL_ON(EqnIsEquLit(pos_lit));
+   FAIL_ON(EqnIsEquLit(neg_lit));
+
+   TB_p bank = pos_lit->bank;
+   Term_p neg_term =
+      BetaNormalizeDB(bank, LambdaEtaReduceDB(bank, neg_lit->lterm));
+   Term_p pos_term =
+      BetaNormalizeDB(bank, LambdaEtaReduceDB(bank, pos_lit->lterm));
+
+   FAIL_ON(!TermIsAppliedFreeVar(neg_term));
+   FAIL_ON(!TermIsAppliedFreeVar(pos_term));
+
+   FAIL_ON(neg_term->arity != 2);
+   FAIL_ON(!TermIsFreeVar(neg_term->args[1]));
+
+   Term_p p_var = neg_term->args[0];
+   FAIL_ON(pos_term->arity != 2);
+   FAIL_ON(pos_term->args[0] != p_var);
+
+   Term_p fp = pos_term->args[1];
+   FAIL_ON(fp->arity != 1);
+   FAIL_ON(fp->f_code <= bank->sig->internal_symbols);
+   FAIL_ON(fp->args[0] != p_var);
+   if(choice_symbols_map)
+   {
+      FAIL_ON(IntMapGetVal(choice_symbols_map, fp->f_code));
+   }
+
+   if(choice_symbols_map)
+   {
+      neg_lit->lterm = neg_term;
+      pos_lit->lterm = pos_term;
+      IntMapAssign(choice_symbols_map, fp->f_code, cl);
+   }
+   return true;
+}
+
 /*---------------------------------------------------------------------*/
 /*                        End of File                                  */
 /*---------------------------------------------------------------------*/
