@@ -146,6 +146,7 @@ ProofState_p ProofStateAlloc(FunctionProperties free_symb_prop)
 
    handle->type_bank            = TypeBankAlloc();
    handle->signature            = SigAlloc(handle->type_bank);
+   handle->choice_opcodes       = IntMapAlloc();
    SigInsertInternalCodes(handle->signature);
    handle->original_symbols     = 0;
    handle->terms                = TBAlloc(handle->signature);
@@ -182,23 +183,22 @@ ProofState_p ProofStateAlloc(FunctionProperties free_symb_prop)
    handle->definition_store     = DefStoreAlloc(handle->terms);
    handle->def_store_cspec      = NULL;
 
-   handle->gc_terms             = GCAdminAlloc(handle->terms);
-   GCRegisterFormulaSet(handle->gc_terms, handle->f_axioms);
-   GCRegisterFormulaSet(handle->gc_terms, handle->f_ax_archive);
-   GCRegisterClauseSet(handle->gc_terms, handle->axioms);
-   GCRegisterClauseSet(handle->gc_terms, handle->ax_archive);
-   GCRegisterClauseSet(handle->gc_terms, handle->processed_pos_rules);
-   GCRegisterClauseSet(handle->gc_terms, handle->processed_pos_eqns);
-   GCRegisterClauseSet(handle->gc_terms, handle->processed_neg_units);
-   GCRegisterClauseSet(handle->gc_terms, handle->processed_non_units);
-   GCRegisterClauseSet(handle->gc_terms, handle->unprocessed);
-   GCRegisterClauseSet(handle->gc_terms, handle->tmp_store);
-   GCRegisterClauseSet(handle->gc_terms, handle->eval_store);
-   GCRegisterClauseSet(handle->gc_terms, handle->archive);
-   GCRegisterClauseSet(handle->gc_terms, handle->watchlist);
-   GCRegisterClauseSet(handle->gc_terms, handle->definition_store->def_clauses);
-   GCRegisterFormulaSet(handle->gc_terms, handle->definition_store->def_archive);
-   GCRegisterFormulaSet(handle->gc_terms, handle->f_archive);
+   TBGCRegisterFormulaSet(handle->terms, handle->f_axioms);
+   TBGCRegisterFormulaSet(handle->terms, handle->f_ax_archive);
+   TBGCRegisterClauseSet(handle->terms, handle->axioms);
+   TBGCRegisterClauseSet(handle->terms, handle->ax_archive);
+   TBGCRegisterClauseSet(handle->terms, handle->processed_pos_rules);
+   TBGCRegisterClauseSet(handle->terms, handle->processed_pos_eqns);
+   TBGCRegisterClauseSet(handle->terms, handle->processed_neg_units);
+   TBGCRegisterClauseSet(handle->terms, handle->processed_non_units);
+   TBGCRegisterClauseSet(handle->terms, handle->unprocessed);
+   TBGCRegisterClauseSet(handle->terms, handle->tmp_store);
+   TBGCRegisterClauseSet(handle->terms, handle->eval_store);
+   TBGCRegisterClauseSet(handle->terms, handle->archive);
+   TBGCRegisterClauseSet(handle->terms, handle->watchlist);
+   TBGCRegisterClauseSet(handle->terms, handle->definition_store->def_clauses);
+   TBGCRegisterFormulaSet(handle->terms, handle->definition_store->def_archive);
+   TBGCRegisterFormulaSet(handle->terms, handle->f_archive);
 
    handle->status_reported              = false;
    handle->answer_count                 = 0;
@@ -212,7 +212,9 @@ ProofState_p ProofStateAlloc(FunctionProperties free_symb_prop)
    handle->backward_subsumed_count      = 0;
    handle->backward_rewritten_count     = 0;
    handle->backward_rewritten_lit_count = 0;
+   handle->rw_count                     = 0;
    handle->generated_count              = 0;
+   handle->aggressive_forward_subsumed_count  = 0;
    handle->generated_lit_count          = 0;
    handle->non_trivial_generated_count  = 0;
    handle->context_sr_count     = 0;
@@ -304,7 +306,7 @@ void ProofStateLoadWatchlist(ProofState_p state,
    }
    else if(!watchlist_filename)
    {
-      GCDeregisterClauseSet(state->gc_terms, state->watchlist);
+      GCDeregisterClauseSet(state->terms->gc, state->watchlist);
       ClauseSetFree(state->watchlist);
       state->watchlist = NULL;
    }
@@ -342,7 +344,7 @@ void ProofStateInitWatchlist(ProofState_p state, OCB_p ocb)
       }
       ClauseSetIndexedInsertClauseSet(state->watchlist, tmpset);
       ClauseSetFree(tmpset);
-      GlobalIndicesInsertClauseSet(&(state->wlindices),state->watchlist);
+      GlobalIndicesInsertClauseSet(&(state->wlindices),state->watchlist, false);
       // ClauseSetPrint(stdout, state->watchlist, true);
    }
 }
@@ -386,7 +388,7 @@ void ProofStateResetClauseSets(ProofState_p state, bool term_gc)
    }
    if(term_gc)
    {
-      GCCollect(state->gc_terms);
+      TBGCCollect(state->terms);
       //GCCollect(state->gc_original_terms);
    }
 }
@@ -408,6 +410,7 @@ void ProofStateFree(ProofState_p junk)
 {
    assert(junk);
    ClauseSetFree(junk->axioms);
+   IntMapFree(junk->choice_opcodes);
    FormulaSetFree(junk->f_axioms);
    FormulaSetFree(junk->f_ax_archive);
    ClauseSetFree(junk->processed_pos_rules);
@@ -422,8 +425,6 @@ void ProofStateFree(ProofState_p junk)
    FormulaSetFree(junk->f_archive);
    PStackFree(junk->extract_roots);
    GlobalIndicesFreeIndices(&(junk->gindices));
-   GCAdminFree(junk->gc_terms);
-   //GCAdminFree(junk->gc_original_terms);
    if(junk->watchlist)
    {
       ClauseSetFree(junk->watchlist);
@@ -619,8 +620,10 @@ void ProofStateStatisticsPrint(FILE* out, ProofState_p state)
            state->backward_rewritten_count);
    fprintf(out, "# Generated clauses                    : %ld\n",
            state->generated_count - state->backward_rewritten_count);
-   fprintf(out, "# ...of the previous two non-trivial   : %ld\n",
+   fprintf(out, "# ...of the previous two non-redundant : %ld\n",
            state->non_trivial_generated_count);
+   fprintf(out, "# ...aggressively subsumed             : %ld\n",
+           state->aggressive_forward_subsumed_count);
    fprintf(out, "# Contextual simplify-reflections      : %ld\n",
            state->context_sr_count);
    fprintf(out, "# Paramodulations                      : %ld\n",
@@ -631,6 +634,8 @@ void ProofStateStatisticsPrint(FILE* out, ProofState_p state)
            state->neg_ext_count);
    fprintf(out, "# Equation resolutions                 : %ld\n",
            state->resolv_count);
+   fprintf(out, "# Total rewrite steps                  : %lu\n",
+           state->rw_count);
    fprintf(out, "# Propositional unsat checks           : %ld\n",
            state->satcheck_count);
    fprintf(out, "#    Propositional check models        : %ld\n",
@@ -708,9 +713,6 @@ void ProofStateStatisticsPrint(FILE* out, ProofState_p state)
               ClauseSetGetTermNodes(state->processed_neg_units)+
               ClauseSetGetTermNodes(state->processed_non_units)+
               ClauseSetGetTermNodes(state->unprocessed));
-      fprintf(out,
-         "# Shared rewrite steps                 : %lu\n",
-              state->terms->rewrite_steps);
       fprintf(out,
               "# Match attempts with oriented units   : %lu\n"
               "# Match attempts with unoriented units : %lu\n",
