@@ -40,12 +40,15 @@
 /*---------------------------------------------------------------------*/
 
 
+
+
+
 /*-----------------------------------------------------------------------
 //
 // Function: forward_contract_keep()
 //
 //   Apply all forward-contracting inferences to clause. Return NULL
-//   if it becomes trivial, a FVPackedClause containing it
+//   if it becomes trivialredundant, a FVPackedClause containing it
 //   otherwise. Does not delete clause. Subsumed and trivial clauses
 //   are counted in the cells pointed to by the 4th and 5th
 //   argument. Provide dummies to avoid this.
@@ -65,7 +68,6 @@ static FVPackedClause_p forward_contract_keep(ProofState_p state, ProofControl_p
                                               RewriteLevel level)
 {
    FVPackedClause_p pclause;
-   Clause_p subsumer = 0;
    bool trivial = false;
 
    assert(clause);
@@ -73,15 +75,27 @@ static FVPackedClause_p forward_contract_keep(ProofState_p state, ProofControl_p
 
    if(control->heuristic_parms.enable_given_forward_simpl)
    {
+      // fprintf(stderr, "before rw: ");
+      // ClausePrint(stderr, clause, true);
+      // fprintf(stderr, ".\n");
       trivial = ForwardModifyClause(state, control, clause,
                                     context_sr, condense, level);
+      // fprintf(stderr, "after rw: ");
+      // ClausePrint(stderr, clause, true);
+      // fprintf(stderr, ".\n");
       if(trivial)
       {
          (*trivial_count)++;
          return NULL;
       }
 
-      if(ClauseIsEmpty(clause) || 
+      if(BooleanSimplification(clause))
+      {
+         (*trivial_count)++;
+         return NULL;
+      }
+
+      if(ClauseIsEmpty(clause) ||
          (problemType == PROBLEM_HO && ResolveFlexClause(clause)))
       {
          return FVIndexPackClause(clause, NULL);
@@ -110,30 +124,18 @@ static FVPackedClause_p forward_contract_keep(ProofState_p state, ProofControl_p
 
       assert(!ClauseIsTrivial(clause));
 
-      clause->weight = ClauseStandardWeight(clause);
-      pclause = FVIndexPackClause(clause, state->processed_non_units->fvindex);
-
-      if(clause->pos_lit_no)
+      if(problemType==PROBLEM_HO)
       {
-         subsumer = UnitClauseSetSubsumesClause(state->processed_pos_eqns, clause);
+         if(ClauseEliminateNakedBooleanVariables(clause))
+         {
+            (*trivial_count)++;
+            return NULL;
+         }
       }
-      if(!subsumer && clause->neg_lit_no)
+      pclause = ForwardSubsumption(state, clause, subsumed_count,
+                                   non_unit_subsumption);
+      if(!pclause)
       {
-         subsumer = UnitClauseSetSubsumesClause(state->processed_neg_units,
-                                                clause);
-      }
-      if(!subsumer && (ClauseLiteralNumber(clause)>1) && non_unit_subsumption)
-      {
-         ClauseSubsumeOrderSortLits(clause);
-         subsumer = ClauseSetSubsumesFVPackedClause(state->processed_non_units, pclause);
-      }
-      if(subsumer)
-      {
-         DocClauseQuote(GlobalOut, OutputLevel, 6, pclause->clause,
-                        "subsumed", subsumer);
-         (*subsumed_count)++;
-         FVUnpackClause(pclause);
-         ENSURE_NULL(pclause);
          return NULL;
       }
       if(context_sr && ClauseLiteralNumber(clause) > 1)
@@ -169,6 +171,61 @@ static FVPackedClause_p forward_contract_keep(ProofState_p state, ProofControl_p
 /*                         Exported Functions                          */
 /*---------------------------------------------------------------------*/
 
+
+/*-----------------------------------------------------------------------
+//
+// Function: ForwadSubsumption()
+//
+//   Try to subsume clause with clauses in state->processed*. Return
+//   NULL if this succeeds, a FVPackedClause containing clause
+//   otherwise. Note that clause is _not_ deleted in either case!
+//
+// Global Variables: -
+//
+// Side Effects    : Sets clause->weight
+//
+/----------------------------------------------------------------------*/
+
+FVPackedClause_p ForwardSubsumption(ProofState_p state,
+                                            Clause_p clause,
+                                            unsigned long* subsumed_count,
+                                            bool non_unit_subsumption)
+{
+   FVPackedClause_p pclause;
+   Clause_p subsumer = NULL;
+
+   clause->weight = ClauseStandardWeight(clause);
+   pclause = FVIndexPackClause(clause, state->processed_non_units->fvindex);
+
+   if(clause->pos_lit_no)
+   {
+      subsumer = UnitClauseSetSubsumesClause(state->processed_pos_eqns, clause);
+   }
+   if(!subsumer && clause->neg_lit_no)
+   {
+      subsumer = UnitClauseSetSubsumesClause(state->processed_neg_units,
+                                             clause);
+   }
+   if(!subsumer && (ClauseLiteralNumber(clause)>1) && non_unit_subsumption)
+   {
+      ClauseSubsumeOrderSortLits(clause);
+      subsumer = ClauseSetSubsumesFVPackedClause(state->processed_non_units, pclause);
+   }
+   if(subsumer)
+   {
+      DocClauseQuote(GlobalOut, OutputLevel, 6, pclause->clause,
+                     "subsumed", subsumer);
+      (*subsumed_count)++;
+      FVUnpackClause(pclause);
+      ENSURE_NULL(pclause);
+      return NULL;
+   }
+   return pclause;
+}
+
+
+
+
 /*-----------------------------------------------------------------------
 //
 // Function: ForwardModifyClause()
@@ -202,12 +259,23 @@ bool ForwardModifyClause(ProofState_p state,
       ClauseQueryProp(clause, CPLimitedRW) did not change any more. */
    while(!done)
    {
-      ClauseComputeLINormalform(control->ocb,
-                                state->terms, clause,
-                                state->demods, level,
-                                control->heuristic_parms.prefer_general);
+      if(problemType == PROBLEM_HO)
+      {
+         NormalizeEquations(clause);
+      }
+      state->rw_count +=
+         ClauseComputeLINormalform(control->ocb,
+                                   state->terms, clause,
+                                   state->demods, level,
+                                   control->heuristic_parms.prefer_general,
+                                   control->heuristic_parms.lambda_demod);
+      if(problemType == PROBLEM_HO)
+      {
+         NormalizeEquations(clause);
+      }
 
       limited_rw = ClauseQueryProp(clause, CPLimitedRW);
+      // DBG_PRINT(stderr, "removing superflous: ", ClausePrintDBG(stderr, clause), ".\n");
       removed_lits = ClauseRemoveSuperfluousLiterals(clause);
       if(removed_lits)
       {
@@ -219,6 +287,15 @@ bool ForwardModifyClause(ProofState_p state,
          ClauseRemoveACResolved(clause);
       }
 
+      if(control->heuristic_parms.local_rw
+         && ClauseLocalRW(clause))
+      {
+         if(problemType == PROBLEM_HO)
+         {
+            NormalizeEquations(clause);
+         }
+      }
+
       /* Now we mark maximal terms... */
       ClauseOrientLiterals(control->ocb, clause);
 
@@ -237,7 +314,17 @@ bool ForwardModifyClause(ProofState_p state,
          return true;
       }
 
+      if(problemType == PROBLEM_HO &&
+         control->heuristic_parms.prune_args)
+      {
+         ClausePruneArgs(clause);
+      }
+
       /* Still forward simplification... */
+      if(problemType == PROBLEM_HO)
+      {
+         NormalizeEquations(clause);
+      }
       if(clause->neg_lit_no)
       {
          ClausePositiveSimplifyReflect(state->processed_pos_eqns, clause);
