@@ -19,6 +19,7 @@
   -----------------------------------------------------------------------*/
 
 #include "che_heuristics.h"
+#include "che_new_autoschedule.h"
 
 
 
@@ -71,16 +72,18 @@ void finalize_auto_parms(char* modename, char* hname,
                          HeuristicParms_p parms,
                          SpecFeature_p spec)
 {
+   control->heuristic_parms = *parms;
 
-
-   if(OutputLevel+1)
+   if(OutputLevel ||
+#ifndef NDEBUG
+false // true
+#else
+false
+#endif   
+   )
    {
-      fprintf(GlobalOut,
-              "# %s selected heuristic %s\n"
-              "# and selection function %s.\n#\n",
-              modename,
-              hname,
-              GetLitSelName(control->heuristic_parms.selection_strategy));
+      fprintf(GlobalOut, "Selected heuristic:\n");
+      HeuristicParmsPrint(stderr, &(control->heuristic_parms));
    }
    if(parms->mem_limit>2 && (parms->delete_bad_limit ==
                              DEFAULT_DELETE_BAD_LIMIT))
@@ -121,6 +124,11 @@ HCB_p HCBCreate(char* name, HCBARGUMENTS)
 
    assert(control->ocb);
 
+   if(strstr(name, "AutoNewSched_"))
+   {
+      return HCBAutoModeCreate(state,control,parms);
+   }
+
    for(i=0; HeuristicsTable[i].heuristic; i++)
    {
       if(strcmp(HeuristicsTable[i].name, name)==0)
@@ -215,18 +223,7 @@ __attribute__((optimize(0)))
 #endif
 HCB_p HCBAutoModeCreate(HCBARGUMENTS)
 {
-   char *res = "Default";
-   SpecFeature_p spec = &(control->problem_specs);
-   SpecLimits_p limits = CreateDefaultSpecLimits();
-
-   control->heuristic_parms.selection_strategy = SelectNoLiterals;
-   OUTPRINT(1, "# Auto-Heuristic is analysing problem.\n");
-#include "che_auto_cases.c"
-   SpecLimitsCellFree(limits);
-
-   finalize_auto_parms("Auto-Mode", res, control, parms, spec);
-
-   return GetHeuristic(res, state, control, parms);
+   return GetHeuristic(parms->heuristic_def, state, control, parms);
 }
 GCC_DIAGNOSTIC_POP
 #undef CHE_HEURISTICS_AUTO
@@ -278,7 +275,6 @@ HCB_p HCBCASCAutoModeCreate(HCBARGUMENTS)
 
    control->heuristic_parms.selection_strategy = SelectNoLiterals;
    OUTPRINT(1, "# Auto-Heuristic is analysing problem.\n");
-#include "che_auto_cases.c"
    SpecLimitsCellFree(limits);
 
    finalize_auto_parms("Auto-Mode (CASC-20)", res, control, parms, spec);
@@ -319,7 +315,7 @@ HCB_p HCBDevAutoModeCreate(HCBARGUMENTS)
 
    control->heuristic_parms.selection_strategy = SelectNoLiterals;
    OUTPRINT(1, "# Auto-Heuristic is analysing problem.\n");
-#include "che_auto_cases.c"
+
    SpecLimitsCellFree(limits);
 
    finalize_auto_parms("AutoDev-Mode", res, control, parms, spec);
@@ -362,7 +358,7 @@ HCB_p HCBAutoSched0Create(HCBARGUMENTS)
 
    control->heuristic_parms.selection_strategy = SelectNoLiterals;
    OUTPRINT(1, "# Auto-Heuristic is analysing problem.\n");
-#include "che_auto_cases.c"
+
    SpecLimitsCellFree(limits);
 
    finalize_auto_parms("AutoSched0-Mode", res, control, parms, spec);
@@ -388,7 +384,7 @@ HCB_p HCBAutoSched1Create(HCBARGUMENTS)
 
    control->heuristic_parms.selection_strategy = SelectNoLiterals;
    OUTPRINT(1, "# Auto-Heuristic is analysing problem.\n");
-#include "che_auto_cases.c"
+
    SpecLimitsCellFree(limits);
 
    finalize_auto_parms("AutoSched1-Mode", res, control, parms, spec);
@@ -414,7 +410,7 @@ HCB_p HCBAutoSched2Create(HCBARGUMENTS)
 
    control->heuristic_parms.selection_strategy = SelectNoLiterals;
    OUTPRINT(1, "# Auto-Heuristic is analysing problem.\n");
-#include "che_auto_cases.c"
+
    SpecLimitsCellFree(limits);
 
    finalize_auto_parms("AutoSched2-Mode", res, control, parms, spec);
@@ -440,7 +436,7 @@ HCB_p HCBAutoSched3Create(HCBARGUMENTS)
 
    control->heuristic_parms.selection_strategy = SelectNoLiterals;
    OUTPRINT(1, "# Auto-Heuristic is analysing problem.\n");
-#include "che_auto_cases.c"
+
    SpecLimitsCellFree(limits);
 
    finalize_auto_parms("AutoSched3-Mode", res, control, parms, spec);
@@ -467,7 +463,7 @@ HCB_p HCBAutoSched4Create(HCBARGUMENTS)
 
    control->heuristic_parms.selection_strategy = SelectNoLiterals;
    OUTPRINT(1, "# Auto-Heuristic is analysing problem.\n");
-#include "che_auto_cases.c"
+
    SpecLimitsCellFree(limits);
 
    finalize_auto_parms("AutoSched4-Mode", res, control, parms, spec);
@@ -494,7 +490,7 @@ HCB_p HCBAutoSched5Create(HCBARGUMENTS)
 
    control->heuristic_parms.selection_strategy = SelectNoLiterals;
    OUTPRINT(1, "# Auto-Heuristic is analysing problem.\n");
-#include "che_auto_cases.c"
+
    SpecLimitsCellFree(limits);
 
    finalize_auto_parms("AutoSched5-Mode", res, control, parms, spec);
@@ -521,7 +517,7 @@ HCB_p HCBAutoSched6Create(HCBARGUMENTS)
 
    control->heuristic_parms.selection_strategy = SelectNoLiterals;
    OUTPRINT(1, "# Auto-Heuristic is analysing problem.\n");
-#include "che_auto_cases.c"
+
    SpecLimitsCellFree(limits);
 
    finalize_auto_parms("AutoSched6-Mode", res, control, parms, spec);
@@ -548,7 +544,7 @@ HCB_p HCBAutoSched7Create(HCBARGUMENTS)
 
    control->heuristic_parms.selection_strategy = SelectNoLiterals;
    OUTPRINT(1, "# Auto-Heuristic is analysing problem.\n");
-#include "che_auto_cases.c"
+
    SpecLimitsCellFree(limits);
 
    finalize_auto_parms("AutoSched7-Mode", res, control, parms, spec);
@@ -576,7 +572,7 @@ HCB_p HCBAutoSched8Create(HCBARGUMENTS)
 
    control->heuristic_parms.selection_strategy = SelectNoLiterals;
    OUTPRINT(1, "# Auto-Heuristic is analysing problem.\n");
-#include "che_auto_cases.c"
+
    SpecLimitsCellFree(limits);
 
    finalize_auto_parms("AutoSched8-Mode", res, control, parms, spec);
@@ -603,7 +599,7 @@ HCB_p HCBAutoSched9Create(HCBARGUMENTS)
 
    control->heuristic_parms.selection_strategy = SelectNoLiterals;
    OUTPRINT(1, "# Auto-Heuristic is analysing problem.\n");
-#include "che_auto_cases.c"
+
    SpecLimitsCellFree(limits);
 
    finalize_auto_parms("AutoSched9-Mode", res, control, parms, spec);
