@@ -32,6 +32,7 @@ Changes
 #include <che_rawspecfeatures.h>
 #include <cco_sine.h>
 #include <e_version.h>
+#include <che_new_autoschedule.h>
 
 /*---------------------------------------------------------------------*/
 /*                  Data types                                         */
@@ -98,7 +99,8 @@ typedef enum
    OPT_FUNC_MEDIUM_LIMIT,
    OPT_FUNC_LARGE_LIMIT,
    OPT_FUN_MEDIUM_LIMIT,
-   OPT_FUN_LARGE_LIMIT
+   OPT_FUN_LARGE_LIMIT,
+   OPT_MERGED_CLASSIFICATION,
 }OptionCodes;
 
 
@@ -462,6 +464,12 @@ OptCell opts[] =
     "Set the minimum number of non-constant function symbols for large size "
     "by this measure."},
 
+   {OPT_MERGED_CLASSIFICATION,
+    '\0', "merged-classification",
+    ReqArg, NULL,
+    "Perform classification that merges formula and clause properties. "
+    "The required argument is the timeout given to clausification (seconds)."},
+
    {OPT_NOOPT,
     '\0', NULL,
     NoArg, NULL,
@@ -469,8 +477,8 @@ OptCell opts[] =
 };
 
 char *outname = NULL,
-     *mask = "aaaaa----aaaa",
-     *raw_mask = "aaaaaaa";
+     *mask = "aaaa-aaaaaa-a",
+     *raw_mask = "aaaaaaaaaa";
 
 IOFormat parse_format     = AutoFormat;
 bool tptp_header      = false,
@@ -480,12 +488,13 @@ bool tptp_header      = false,
    new_cnf          = true,
    parse_features   = false,
    app_encode       = false;
+int cnf_timeout = -1;
 long eqdef_maxclauses = DEFAULT_EQDEF_MAXCLAUSES,
    miniscope_limit  = 1000;
 long eqdef_incrlimit  = DEFAULT_EQDEF_INCRLIMIT;
 FunctionProperties free_symb_prop = FPIgnoreProps;
-ProblemType problemType  = PROBLEM_NOT_INIT;
 char *sine = NULL;
+long FormulaDefLimit = 24;
 
 /*---------------------------------------------------------------------*/
 /*                      Forward Declarations                           */
@@ -804,7 +813,7 @@ int main(int argc, char* argv[])
 
    InitIO(NAME);
 
-   limits = SpecLimitsAlloc();
+   limits = CreateDefaultSpecLimits();
    state = process_options(argc, argv, limits);
 
    OpenGlobalOut(outname);
@@ -837,12 +846,25 @@ int main(int argc, char* argv[])
                                   fstate->terms, NULL, &skip_includes);
          ProofStateSinE(fstate, sine);
 
-         if(raw_classify)
+         if(cnf_timeout != -1)
+         {
+            // merged classification
+            RawSpecFeatureCell features;
+            RawSpecFeaturesCompute(&features, fstate);
+            RawSpecFeaturesClassify(&features, limits, RAW_DEFAULT_MASK);
+            char cnf_class[SPEC_STRING_MEM];
+            ClausifyAndClassifyWTimeout(fstate, cnf_timeout, DEFAULT_MASK, cnf_class);
+            fprintf(GlobalOut, "%s : (NULL) : %s%s\n", state->argv[i], features.class, cnf_class);
+         }
+         else if(raw_classify)
          {
             do_raw_classification(state->argv[i], fstate, limits);
          }
          else
          {
+            RawSpecFeatureCell raw_features;
+            RawSpecFeaturesCompute(&raw_features, fstate);
+            RawSpecFeaturesClassify(&raw_features, limits, RAW_DEFAULT_MASK);
             FormulaSetPreprocConjectures(fstate->f_axioms, fstate->f_ax_archive,
                                          false, false);
             if(new_cnf)
@@ -852,8 +874,8 @@ int main(int argc, char* argv[])
                               fstate->axioms,
                               fstate->terms,
                               fstate->freshvars,
-                              fstate->gc_terms,
-                              miniscope_limit);
+                              miniscope_limit, FormulaDefLimit,
+                              true, true, true, true);
             }
             else
             {
@@ -862,7 +884,7 @@ int main(int argc, char* argv[])
                              fstate->axioms,
                              fstate->terms,
                              fstate->freshvars,
-                             fstate->gc_terms);
+                             FormulaDefLimit);
             }
             if(!no_preproc)
             {
@@ -877,7 +899,12 @@ int main(int argc, char* argv[])
             }
             if(!specsig_classify)
             {
-               SpecFeaturesCompute(&features, fstate->axioms, fstate->signature);
+               SpecFeaturesCompute(&features, fstate->axioms, fstate->f_axioms,
+                                   fstate->f_ax_archive, fstate->terms);
+               features.order = raw_features.order;
+               features.goal_order = raw_features.conj_order;
+               features.num_of_definitions = raw_features.num_of_definitions;
+               features.perc_of_form_defs = raw_features.perc_of_form_defs;
                SpecFeaturesAddEval(&features, limits);
 
                if(!tptp_header)
@@ -1035,17 +1062,17 @@ CLState_p process_options(int argc, char* argv[], SpecLimits_p limits)
             break;
       case OPT_MASK:
        mask = arg;
-       if(strlen(mask)!=13)
+       if(strlen(mask)<13)
        {
-          Error("Option -c (--class-mask) requires 13-letter "
+          Error("Option -c (--class-mask) requires at least 13-letter "
                 "string as an argument", USAGE_ERROR);
        }
        break;
       case OPT_RAW_MASK:
             raw_mask = arg;
-            if(strlen(raw_mask)!=7)
+            if(strlen(raw_mask)<11)
             {
-               Error("Option -c (--class-mask) requires 7-letter "
+               Error("Option -c (--class-mask) requires at least 11-letter "
                      "string as an argument", USAGE_ERROR);
             }
             break;
@@ -1127,6 +1154,9 @@ CLState_p process_options(int argc, char* argv[], SpecLimits_p limits)
       case OPT_FUN_LARGE_LIMIT:
             limits->fun_large_limit = CLStateGetIntArg(handle, arg);
             break;
+      case OPT_MERGED_CLASSIFICATION:
+            cnf_timeout = CLStateGetIntArg(handle, arg);
+            break;
       default:
             assert(false);
             break;
