@@ -43,6 +43,7 @@ typedef enum
    OPT_PRINT_SATURATED,
    OPT_PRINT_SAT_INFO,
    OPT_FILTER_SATURATED,
+   OPT_SYNTAX_ONLY,
    OPT_PRUNE_ONLY,
    OPT_CNF_ONLY,
    OPT_PRINT_PID,
@@ -51,6 +52,7 @@ typedef enum
    OPT_SILENT,
    OPT_OUTPUTLEVEL,
    OPT_PROOF_OBJECT,
+   OPT_PROOF_STATS,
    OPT_PROOF_GRAPH,
    OPT_FULL_DERIV,
    OPT_FORCE_DERIV,
@@ -64,6 +66,7 @@ typedef enum
    OPT_SOFTCPU_LIMIT,
    OPT_RUSAGE_INFO,
    OPT_PRINT_STRATEGY,
+   OPT_PARSE_STRATEGY,
    OPT_STEP_LIMIT,
    OPT_PROC_LIMIT,
    OPT_UNPROC_LIMIT,
@@ -83,16 +86,15 @@ typedef enum
    OPT_TSTP_PRINT,
    OPT_TSTP_FORMAT,
    OPT_AUTO,
-   OPT_SATAUTO,
-   OPT_AUTODEV,
-   OPT_SATAUTODEV,
    OPT_AUTO_SCHED,
+   OPT_FORCE_PREPROC_SCHED,
    OPT_SATAUTO_SCHED,
-   OPT_AUTOSCHEDULE_KIND,
    OPT_NO_PREPROCESSING,
    OPT_EQ_UNFOLD_LIMIT,
    OPT_EQ_UNFOLD_MAXCLAUSES,
    OPT_NO_EQ_UNFOLD,
+   OPT_INTRO_GOAL_DEFS,
+   OPT_FINE_GOAL_DEFS,
    OPT_SINE,
    OPT_REL_PRUNE_LEVEL,
    OPT_PRESAT_SIMPLIY,
@@ -153,6 +155,7 @@ typedef enum
    OPT_BACKWARD_CSR,
    OPT_RULES_GENERAL,
    OPT_FORWARD_DEMOD,
+   OPT_DEMOD_LAMBDA,
    OPT_STRONG_RHS_INSTANCE,
    OPT_STRONGSUBSUMPTION,
    OPT_SAT_STEP_INTERVAL,
@@ -165,6 +168,7 @@ typedef enum
    OPT_WATCHLIST,
    OPT_STATIC_WATCHLIST,
    OPT_WATCHLIST_NO_SIMPLIFY,
+   OPT_FW_SUMBSUMPTION_AGGRESSIVE,
    OPT_NO_INDEXED_SUBSUMPTION,
    OPT_FVINDEX_STYLE,
    OPT_FVINDEX_FEATURETYPES,
@@ -186,6 +190,7 @@ typedef enum
    OPT_FREE_OBJECTS,
    OPT_DEF_CNF_OLD,
    OPT_DEF_CNF,
+   OPT_FOOL_UNROLL,
    OPT_MINISCOPE_LIMIT,
    OPT_PRINT_TYPES,
    OPT_APP_ENCODE,
@@ -195,6 +200,39 @@ typedef enum
    OPT_EXT_SUP,
    OPT_INVERSE_RECOGNITION,
    OPT_REPLACE_INJ_DEFS,
+   OPT_BCE,
+   OPT_BCE_MAX_OCCS,
+   OPT_PRED_ELIM,
+   OPT_PRED_ELIM_GATES,
+   OPT_PRED_ELIM_FORCE_MU_DECREASE,
+   OPT_PRED_ELIM_IGNORE_CONJ_SYMS,
+   OPT_PRED_ELIM_MAX_OCCS,
+   OPT_PRED_ELIM_TOLERANCE,
+   OPT_LIFT_LAMBDAS,
+   OPT_ETA_NORMALIZE,
+   OPT_HO_ORDER_KIND,
+   OPT_LAMBDA_TO_FORALL,
+   OPT_LAM_WEIGHT,
+   OPT_DB_WEIGHT,
+   OPT_ELIM_LEIBNIZ,
+   OPT_UNROLL_FORMULAS_ONLY,
+   OPT_PRIM_ENUM_KIND,
+   OPT_PRIM_ENUM_MAX_DEPTH,
+   OPT_CHOICE_INST,
+   OPT_LOCAL_RW,
+   OPT_PRUNE_ARGS,
+   OPT_FUNC_PROJ_IMIT,
+   OPT_IMIT_LIMIT,
+   OPT_IDENT_LIMIT,
+   OPT_ELIM_LIMIT,
+   OPT_UNIF_MODE,
+   OPT_PATTERN_ORACLE,
+   OPT_FIXPOINT_ORACLE,
+   OPT_MAX_UNIFIERS,
+   OPT_MAX_UNIF_STEPS,
+   OPT_CNF_TIMEOUT_PORTION,
+   OPT_PREINSTANTIATE_INDUCTION,
+   OPT_SERIALIZE_SCHEDULE,
    OPT_DUMMY
 }OptionCodes;
 
@@ -264,6 +302,11 @@ OptCell opts[] =
     "following option will suppress normal output of the proof "
     "object in favour of a graphial representation."},
 
+   {OPT_PROOF_STATS,
+    '\0', "proof-statistics",
+    NoArg, NULL,
+    "Print various statistics of the proof object."},
+
    {OPT_PROOF_GRAPH,
     '\0', "proof-graph",
     OptArg, "3",
@@ -286,7 +329,7 @@ OptCell opts[] =
     "Force output of the derivation even in cases where the prover "
     "terminates in an indeterminate state. By default, the deriviation "
     "of all processed clauses is included in the derivation object. "
-    "With value 2, derivation of all clauses will be printed"},
+    "With argument 2, the derivation of all clauses will be printed."},
 
    {OPT_RECORD_GIVEN_CLAUSES,
     '\0', "record-gcs",
@@ -301,7 +344,7 @@ OptCell opts[] =
     OptArg, "1",
     "Generate and process training examples from the proof search object."
     " Implies --record-gcs. The argument is a binary or of the desired "
-    "processig. Bit zero prints positive exampels. Bit 1 prints negative "
+    "processing. Bit zero prints positive exampels. Bit 1 prints negative "
     "examples. Additional selectors will be added later."},
 
    {OPT_PCL_COMPRESSED,
@@ -326,7 +369,7 @@ OptCell opts[] =
    {OPT_PRINT_STATISTICS,
     '\0', "print-statistics",
     NoArg, NULL,
-    "Print the inference statistics (only relevant for output level 0, "
+    "Print the inference statistics (only relevant for output level <=1, "
     "otherwise they are printed automatically."},
 
    {OPT_EXPENSIVE_DETAILS,
@@ -372,6 +415,12 @@ OptCell opts[] =
     " lower case counterparts, but with non-unit-subsumption enabled"
     " as well)."},
 
+   {OPT_SYNTAX_ONLY,
+    '\0', "syntax-only",
+    NoArg, NULL,
+    "Stop after parsing, i.e. only check if the input can be parsed "
+    "correcly."},
+
    {OPT_PRUNE_ONLY,
     '\0', "prune",
     NoArg, NULL,
@@ -471,6 +520,12 @@ OptCell opts[] =
     "Print a representation of all search parameters and their setting. "
     "Then terminate."},
 
+   {OPT_PARSE_STRATEGY,
+    '\0', "parse-strategy",
+    ReqArg, NULL,
+    "Parse the previously printed representation of strategy and set"
+    " all proof search parameters accordingly."},
+
    {OPT_STEP_LIMIT,
     'C', "processed-clauses-limit",
     ReqArg, NULL,
@@ -619,48 +674,35 @@ OptCell opts[] =
    {OPT_AUTO,
     '\0', "auto",
     NoArg, NULL,
-    "Automatically determine settings for proof search. This is "
-    "equivalent to -xAuto -tAuto --sine=Auto."},
-
-   {OPT_SATAUTO,
-    '\0', "satauto",
-    NoArg, NULL,
-    "Automatically determine settings for proof/saturation search. This is "
-    "equivalent to -xAuto -tAuto."},
-
-   {OPT_AUTODEV,
-    '\0', "autodev",
-    NoArg, NULL,
-    "Automatically determine settings for proof search (development "
-    "version). This is equivalent to -xAutoDev -tAutoDev --sine=Auto."},
-
-   {OPT_SATAUTODEV,
-    '\0', "satautodev",
-    NoArg, NULL,
-    "Automatically determine settings for proof/saturation search "
-    "(development version). This is equivalent to -xAutoDev -tAutoDev."},
+    "Automatically determine settings for proof search."},
 
    {OPT_AUTO_SCHED,
     '\0', "auto-schedule",
-    NoArg, NULL,
+    OptArg, "1",
     "Use the (experimental) strategy scheduling. This will try several "
     "different fully specified search strategies (aka \"Auto-Modes\"), "
     "one after the other, until a proof or saturation is found, or the "
-    "time limit is exceeded."},
+    "time limit is exceeded. Optional argument is the number of CPUs "
+    "on which the schedule is going to be executed on. By default, "
+    "the schedule is executed on a single core. To execute on all "
+    "cores of a system set the argument to 'Auto', but note that this"
+    "will use all reported cores (even efficiency cores, if available)."},
+
+    {OPT_FORCE_PREPROC_SCHED,
+    '\0', "force-preproc-sched",
+    ReqArg, NULL,
+    "When autoscheduling is used, make sure that preprocessing schedule is "
+    "inserted in the search categories"},
+
+    {OPT_SERIALIZE_SCHEDULE,
+    '\0', "serialize-schedule",
+    ReqArg, NULL,
+    "Convert parallel auto-schedule into serialized one."},
 
    {OPT_SATAUTO_SCHED,
     '\0', "satauto-schedule",
-    NoArg, NULL,
-    "Use the (experimental) strategy scheduling without SInE, thus "
-    "maintaining completeness."},
-
-   {OPT_AUTOSCHEDULE_KIND,
-    '\0', "schedule-kind",
-    ReqArg, NULL,
-    "Choose a schedule kind that is more optimized for different purposes: "
-    "CASC is optimized for general-purpose theorem proving, while SH "
-    "is optimized for theorem low-timeout theorem proving."
-    },
+    OptArg, "1",
+    "Use strategy scheduling without SInE, thus maintaining completeness."},
 
    {OPT_NO_PREPROCESSING,
     '\0', "no-preprocessing",
@@ -691,6 +733,20 @@ OptCell opts[] =
     "During preprocessing, abstain from unfolding (and removing) "
     "equational definitions."},
 
+   {OPT_INTRO_GOAL_DEFS,
+    '\0', "goal-defs",
+    OptArg, "All",
+    "Introduce Twee-style equational definitions for ground terms "
+    "in conjecture clauses. The argument can be All or Neg, which will"
+    " only consider ground terms from negative literals (to be implemented)."},
+
+   {OPT_FINE_GOAL_DEFS,
+    '\0', "goal-subterm-defs",
+    NoArg, NULL,
+    "Introduce goal definitions for all conjecture ground subterms. "
+    "The default is to only introduce them for the maximal (with respect "
+    "to the subterm relation) ground terms in conjecture clauses (to be implemented)."},
+
    {OPT_SINE,
     '\0', "sine",
     OptArg, "Auto",
@@ -1022,7 +1078,7 @@ OptCell opts[] =
     "Set a weight for symbols that occur in conjectures only to determine"
     "where to place it in the precedence. This value is used for a rough"
     "pre-order, the normal schemes only sort within symbols with the same"
-    "occurance modifier."},
+    "occurrence modifier."},
 
    {OPT_TO_CONJAXIOM_PREC,
     '\0', "prec-conj-axiom",
@@ -1030,7 +1086,7 @@ OptCell opts[] =
     "Set a weight for symbols that occur in both conjectures and axioms"
     "to determine where to place it in the precedence. This value is "
     "used for a rough pre-order, the normal schemes only sort within "
-    "symbols with the same occurance modifier."},
+    "symbols with the same occurrence modifier."},
 
    {OPT_TO_AXIOMONLY_PREC,
     '\0', "prec-pure-axiom",
@@ -1038,7 +1094,7 @@ OptCell opts[] =
     "Set a weight for symbols that occur in axioms only "
     "to determine where to place it in the precedence. This value is "
     "used for a rough pre-order, the normal schemes only sort within "
-    "symbols with the same occurance modifier."},
+    "symbols with the same occurrence modifier."},
 
    {OPT_TO_SKOLEM_PREC,
     '\0', "prec-skolem",
@@ -1046,7 +1102,7 @@ OptCell opts[] =
     "Set a weight for Skolem symbols "
     "to determine where to place it in the precedence. This value is "
     "used for a rough pre-order, the normal schemes only sort within "
-    "symbols with the same occurance modifier."},
+    "symbols with the same occurrence modifier."},
 
    {OPT_TO_DEFPRED_PREC,
     '\0', "prec-defpred",
@@ -1055,7 +1111,7 @@ OptCell opts[] =
     "CNF or clause splitting) "
     "to determine where to place it in the precedence. This value is "
     "used for a rough pre-order, the normal schemes only sort within "
-    "symbols with the same occurance modifier."},
+    "symbols with the same occurrence modifier."},
 
    {OPT_TO_CONSTWEIGHT,
     'c', "order-constant-weight",
@@ -1108,7 +1164,8 @@ OptCell opts[] =
     "Modify how literal comparisons are done. 'None' is equivalent to the "
     "previous option, 'Normal' uses the normal lifting of the term ordering, "
     "'TFOEqMax' uses the equivalent of a transfinite ordering deciding on the "
-    "predicate symbol and making equational literals maximal, and 'TFOEqMin' "
+    "predicate symbol and making equational literals maximal (note that this "
+    "setting makes the prover incomplere), and 'TFOEqMin' "
     "modifies this by making equational symbols minimal."},
 
    {OPT_TPTP_SOS,
@@ -1178,6 +1235,11 @@ OptCell opts[] =
     "(orientable equations) only, 2 indicates full rewriting with "
     "rules and instances of unorientable equations. Default behavior is 2."},
 
+   {OPT_DEMOD_LAMBDA,
+    '\0', "demod-under-lambda",
+    ReqArg, NULL,
+    "Demodulate *closed* subterms under lambdas."},
+
    {OPT_STRONG_RHS_INSTANCE,
     '\0', "strong-rw-inst",
     NoArg, NULL,
@@ -1268,6 +1330,13 @@ OptCell opts[] =
     "to the current processed clause set and certain simplifications. "
     "This option disables simplification for the watchlist."},
 
+   {OPT_FW_SUMBSUMPTION_AGGRESSIVE,
+    '\0', "fw-subsumption-aggressive",
+    NoArg, NULL,
+    "Perform forward subsumption on newly generated clauses before they "
+    "are evaluated. This is particularly useful if heuristic evaluation is "
+    "very expensive, e.g. via externally connected neural networks."},
+
    {OPT_NO_INDEXED_SUBSUMPTION,
     '\0', "conventional-subsumption",
     NoArg, NULL,
@@ -1415,6 +1484,11 @@ OptCell opts[] =
     "subformulae to avoid exponential blow-up. The optional argument "
     "is a fudge factor that determines when definitions are introduced. "
     "0 disables definitions completely. The default works well."},
+    // OPT_FOOL_UNROLL
+   {OPT_FOOL_UNROLL,
+    '\0', "fool-unroll",
+    ReqArg, NULL,
+    "Enable or disable FOOL unrolling. Useful for some SH problems."},
 
    {OPT_DEF_CNF_OLD,
     '\0', "old-cnf",
@@ -1469,7 +1543,7 @@ OptCell opts[] =
     '\0', "ext-sup-max-depth",
     ReqArg, NULL,
     "Sets the maximal proof depth of the clause which will be considered for "
-    " ExtSup inferences. Negative value disables the rule."},
+    " Ext-family of inferences. Negative value disables the rule."},
 
    {OPT_INVERSE_RECOGNITION,
     '\0', "inverse-recognition",
@@ -1484,6 +1558,171 @@ OptCell opts[] =
     "After CNF and before saturation, replaces all clauses that are definitions "
     " of injectivity by axiomatization of inverse function."},
 
+   {OPT_LIFT_LAMBDAS,
+    '\0', "lift-lambdas",
+    ReqArg, NULL,
+    "Should the lambdas be replaced by named fuctions?"},
+
+   {OPT_ETA_NORMALIZE,
+    '\0', "eta-normalize",
+    ReqArg, NULL,
+    "Which form of eta normalization to perform?"},
+
+   {OPT_HO_ORDER_KIND,
+    '\0', "ho-order-kind",
+    ReqArg, NULL,
+    "Do we use simple LFHO order or a more advanced Boolean free lambda-KBO?"},
+
+   {OPT_LAMBDA_TO_FORALL,
+    '\0', "cnf-lambda-to-forall",
+    ReqArg, NULL,
+    "Do we turn equations of the form ^X.s (!)= ^X.t into (?)!X. s (!)= t ?"},
+
+   {OPT_LAM_WEIGHT,
+    '\0', "kbo-lam-weight",
+    ReqArg, NULL,
+    "Weight of lambda symbol in KBO."},
+
+   {OPT_DB_WEIGHT,
+    '\0', "kbo-db-weight",
+    ReqArg, NULL,
+    "Weight of DB var in KBO."},
+
+   {OPT_ELIM_LEIBNIZ,
+    '\0', "eliminate-leibniz-eq",
+    ReqArg, NULL,
+    "Maximal proof depth of the clause on which Leibniz"\
+    " equality elimination should be applied"\
+    "; -1 disaables Leibniz equality elimination altogether"},
+
+    {OPT_UNROLL_FORMULAS_ONLY,
+    '\0', "unroll-formulas-only",
+    ReqArg, NULL,
+    "Set to true if you want only formulas to be recognized as definitions"
+    " during CNF. Default is true."},
+
+    {OPT_PRIM_ENUM_KIND,
+    '\0', "prim-enum-mode",
+    ReqArg, NULL,
+    "Choose the mode of primitive enumeration "},
+
+    {OPT_PRIM_ENUM_MAX_DEPTH,
+    '\0', "prim-enum-max-depth",
+    ReqArg, NULL,
+    "Maximal proof depth of a clause on which primitive enumeration is applied."
+    " -1 disables primitive enumeration"},
+
+    {OPT_CHOICE_INST,
+    '\0', "inst-choice-max-depth",
+    ReqArg, NULL,
+    "Maximal proof depth of a clause which is going to"
+    " be scanned for occurrences of defined choice symbol"
+    " -1 disables scanning for choice symbols"},
+
+    {OPT_LOCAL_RW,
+    '\0', "local-rw",
+    ReqArg, NULL,
+    "Enable/disable local rewriting: if the clause is of the form s != t | "
+    " C, where s > t, rewrite all occurrences of s with t in C."},
+
+    {OPT_PRUNE_ARGS,
+    '\0', "prune-args",
+    ReqArg, NULL,
+    "Enable/disable pruning arguments of applied variables."},
+
+    {OPT_FUNC_PROJ_IMIT,
+    '\0', "func-proj-limit",
+    ReqArg, NULL,
+    "Maximal number of functional projections"},
+
+    {OPT_IMIT_LIMIT,
+    '\0', "imit-limit",
+    ReqArg, NULL,
+    "Maximal number of imitations"},
+
+    {OPT_IDENT_LIMIT,
+    '\0', "ident-limit",
+    ReqArg, NULL,
+    "Maximal number of identifications"},
+
+    {OPT_ELIM_LIMIT,
+    '\0', "elim-limit",
+    ReqArg, NULL,
+    "Maximal number of eliminations"},
+
+    {OPT_UNIF_MODE,
+    '\0', "unif-mode",
+    ReqArg, NULL,
+    "Set the mode of unification: either single or multi."},
+
+    {OPT_PATTERN_ORACLE,
+    '\0', "pattern-oracle",
+    ReqArg, NULL,
+    "Turn the pattern oracle on or off."},
+
+    {OPT_FIXPOINT_ORACLE,
+    '\0', "fixpoint-oracle",
+    ReqArg, NULL,
+    "Turn the pattern oracle on or off."},
+
+    {OPT_MAX_UNIFIERS,
+    '\0', "max-unifiers",
+    ReqArg, NULL,
+    "Maximal number of imitations"},
+
+    {OPT_MAX_UNIF_STEPS,
+    '\0', "max-unif-steps",
+    ReqArg, NULL,
+    "Maximal number of variable bindings that can "
+    "be done in one single call to copmuting the next unifier."},
+
+    {OPT_CNF_TIMEOUT_PORTION,
+    '\0', "classification-timeout-portion",
+    ReqArg, NULL,
+    "Which percentage (from 1 to 99) of the total CPU time will be devoted to "
+    "problem classification?"},
+
+    {OPT_PREINSTANTIATE_INDUCTION,
+    '\0', "preinstantiate-induction",
+    ReqArg, NULL,
+    "Abstract unit clauses coming from conjecture and use the abstractions "
+    "to instantiate clauses that look like the ones coming from induction axioms."},
+
+    {OPT_BCE,
+    '\0', "bce", ReqArg, NULL, "Turn blocked clause elimination on or off"},
+
+    {OPT_BCE_MAX_OCCS,
+    '\0', "bce-max-occs", ReqArg, NULL,
+    "Stop tracking symbol after it occurs in <arg> clauses"
+    " Set <arg> to -1 disable this limit"},
+
+    {OPT_PRED_ELIM,
+    '\0', "pred-elim", ReqArg, NULL, "Turn predicate elimination on or off"},
+
+    {OPT_PRED_ELIM_MAX_OCCS,
+    '\0', "pred-elim-max-occs", ReqArg, NULL,
+    "Stop tracking symbol after it occurs in <arg> clauses"
+    " Set <arg> to -1 disable this limit"},
+
+    {OPT_PRED_ELIM_TOLERANCE,
+    '\0', "pred-elim-tolerance", ReqArg, NULL,
+    "Tolerance for predicate elimination measures."},
+
+    {OPT_PRED_ELIM_GATES,
+    '\0', "pred-elim-recognize-gates", ReqArg, NULL, "Turn gate recognition for predicate elimination on or off"},
+
+    {OPT_PRED_ELIM_FORCE_MU_DECREASE,
+    '\0', "pred-elim-force-mu-decrease", ReqArg, NULL,
+     "Require that the square number of distinct free variables "
+     "decreases when doing predicate elimination. Helps avoid "
+     "creating huge clauses."},
+
+    {OPT_PRED_ELIM_IGNORE_CONJ_SYMS,
+    '\0', "pred-elim-ignore-conj-syms", ReqArg, NULL,
+     "Disable eliminating symbols that occur in the conjecture."},
+
+
+
    {OPT_NOOPT,
     '\0', NULL,
     NoArg, NULL,
@@ -1491,7 +1730,6 @@ OptCell opts[] =
 };
 
 #endif
-
 /*---------------------------------------------------------------------*/
 /*                        End of File                                  */
 /*---------------------------------------------------------------------*/
