@@ -1,10 +1,10 @@
 /*-----------------------------------------------------------------------
 
-File  : enormalizer.c
+  File  : enormalizer.c
 
-Author: Stephan Schulz
+  Author: Stephan Schulz
 
-Contents
+  Contents
 
   Read a set of unit clauses (and/or formulas) and a set of
   terms/clauses/formulas. The unit clauses/formulas are interpreted as
@@ -13,16 +13,13 @@ Contents
   deterministic but unspecified. If the rule system is not
   terminating, rewriting might get stuck into an infinite loop.
 
-  Copyright 2013 by the author.
+  Copyright 2013-2022 by the author.
   This code is released under the GNU General Public Licence and
   the GNU Lesser General Public License.
   See the file COPYING in the main E directory for details..
   Run "eprover -h" for contact information.
 
-Changes
-
-<1> Mon Feb  4 23:21:45 CET 2013
-    New
+  Created: Mon Feb  4 23:21:45 CET 2013
 
 -----------------------------------------------------------------------*/
 
@@ -264,7 +261,6 @@ bool   print_statistics = false,
 long   give_up = 0,
        initial_literals = 0,
        initial_clauses = 0;
-ProblemType problemType  = PROBLEM_NOT_INIT;
 
 /*---------------------------------------------------------------------*/
 /*                      Forward Declarations                           */
@@ -343,7 +339,7 @@ void process_terms(char* infile, TB_p terms, OCB_p ocb, ClauseSet_p *demodulator
          t  = TBTermParse(in, terms);
          tp = TermComputeLINormalform(ocb, terms, t,
                                       demodulators,
-                                      1, false, false);
+                                      1, false, false, false);
          TBPrintTermFull(GlobalOut, terms, t);
          fprintf(GlobalOut, " ==> ");
          TBPrintTermFull(GlobalOut, terms, tp);
@@ -381,7 +377,7 @@ void process_clauses(char* infile, TB_p terms, OCB_p ocb, ClauseSet_p *demodulat
          ClausePrint(GlobalOut, clause, true);
          ClauseComputeLINormalform(ocb, terms, clause,
                                       demodulators,
-                                      1, false);
+                                      1, false, false);
          fprintf(GlobalOut, " ==> ");
          ClausePrint(GlobalOut, clause, true);
          fprintf(GlobalOut, "\n");
@@ -420,7 +416,7 @@ void process_formulas(char* infile, TB_p terms, OCB_p ocb, ClauseSet_p *demodula
 
          form->tformula = TermComputeLINormalform(ocb, terms, form->tformula,
                                                   demodulators,
-                                                  1, false, false);
+                                                  1, false, false, false);
          fprintf(GlobalOut, " ==> ");
          WFormulaPrint(GlobalOut, form, true);
          fprintf(GlobalOut, "\n");
@@ -446,7 +442,6 @@ void process_formulas(char* infile, TB_p terms, OCB_p ocb, ClauseSet_p *demodula
 int main(int argc, char* argv[])
 {
    TB_p            terms;
-   GCAdmin_p       collector;
    VarBank_p       freshvars;
    TypeBank_p      typebank;
    Sig_p           sig;
@@ -479,15 +474,14 @@ int main(int argc, char* argv[])
    sig          = SigAlloc(typebank);
    SigInsertInternalCodes(sig);
    terms        = TBAlloc(sig);
-   collector    = GCAdminAlloc(terms);
    clauses      = ClauseSetAlloc();
    dummy        = ClauseSetAlloc();
    formulas     = FormulaSetAlloc();
    f_ax_archive = FormulaSetAlloc();
 
-   GCRegisterClauseSet(collector, clauses);
-   GCRegisterFormulaSet(collector, formulas);
-   GCRegisterFormulaSet(collector, f_ax_archive);
+   TBGCRegisterClauseSet(terms, clauses);
+   TBGCRegisterFormulaSet(terms, formulas);
+   TBGCRegisterFormulaSet(terms, f_ax_archive);
 
    for(i=0; state->argv[i]; i++)
    {
@@ -509,31 +503,31 @@ int main(int argc, char* argv[])
    freshvars = VarBankAlloc(typebank);
    if(FormulaSetCNF2(formulas, f_ax_archive,
                      clauses, terms, freshvars,
-                     collector, 1000))
+                     1000, 24, true, true, true, true))
    {
       VERBOUT("CNFization done\n");
    }
    VarBankFree(freshvars);
 
-   GCDeregisterFormulaSet(collector, formulas);
+   GCDeregisterFormulaSet(terms->gc, formulas);
    FormulaSetFree(formulas);
-   GCDeregisterFormulaSet(collector, f_ax_archive);
+   GCDeregisterFormulaSet(terms->gc, f_ax_archive);
    FormulaSetFree(f_ax_archive);
 
    demodulators[0] = ClauseSetAlloc();
    demodulators[0]->demod_index = PDTreeAlloc(terms);
-   GCRegisterClauseSet(collector, demodulators[0]);
+   TBGCRegisterClauseSet(terms, demodulators[0]);
 
    build_rw_system(demodulators[0], clauses);
 
 
-   GCDeregisterClauseSet(collector, clauses);
+   GCDeregisterClauseSet(terms->gc, clauses);
    ClauseSetFree(clauses);
 
    VERBOUT("# Demodulators\n");
    VERBOSE(ClauseSetPrint(stderr, demodulators[0], true););
 
-   ocb = OCBAlloc(EMPTY, false, terms->sig);
+   ocb = OCBAlloc(EMPTY, false, terms->sig, LFHO_ORDER);
 
    process_terms(termname, terms, ocb, demodulators);
    process_clauses(clausename, terms, ocb, demodulators);
@@ -543,7 +537,6 @@ int main(int argc, char* argv[])
 
 #ifndef FAST_EXIT
    ClauseSetFree(demodulators[0]);
-   GCAdminFree(collector);
 
    terms->sig = NULL;
    TBFree(terms);
@@ -713,7 +706,7 @@ CLState_p process_options(int argc, char* argv[])
       if(getrlimit(RLIMIT_CPU, &limit))
       {
          TmpErrno = errno;
-         SysError("Unable to get sytem cpu time limit", SYS_ERROR);
+         SysError("Unable to get system cpu time limit", SYS_ERROR);
       }
       SystemTimeLimit = limit.rlim_max;
       if(SoftTimeLimit!=RLIM_INFINITY)
