@@ -479,11 +479,10 @@ bool tptp_header      = false,
    parse_features   = false,
    app_encode       = false;
 long eqdef_maxclauses = DEFAULT_EQDEF_MAXCLAUSES,
-   miniscope_limit  = 1000;
+   miniscope_limit  = 1000,
+   FormulaDefLimit = 24;
 long eqdef_incrlimit  = DEFAULT_EQDEF_INCRLIMIT;
-FunctionProperties free_symb_prop = FPIgnoreProps;
-ProblemType problemType  = PROBLEM_NOT_INIT;
-char *sine = NULL;
+FunctionProperties free_symb_prop = FPIgnoreProps;char *sine = NULL;
 
 /*---------------------------------------------------------------------*/
 /*                      Forward Declarations                           */
@@ -563,8 +562,8 @@ int main(int argc, char* argv[])
                            fstate->axioms,
                            fstate->terms,
                            fstate->freshvars,
-                           fstate->gc_terms,
-                           miniscope_limit);
+                           miniscope_limit, FormulaDefLimit,
+                           true, true, true, true);
          }
          else
          {
@@ -573,7 +572,7 @@ int main(int argc, char* argv[])
                              fstate->axioms,
                              fstate->terms,
                              fstate->freshvars,
-                             fstate->gc_terms);
+                             FormulaDefLimit);
          }
 
          for(clause = fstate->axioms->anchor->succ;
