@@ -128,6 +128,7 @@ Sig_p SigAlloc(TypeBank_p bank)
    handle->ac_axioms = PStackAlloc();
 
    handle->type_bank = bank;
+   handle->typed_symbols = false;
 
    SigInsertId(handle, "$true", 0, true);
    assert(SigFindFCode(handle, "$true")==SIG_TRUE_CODE);
@@ -213,7 +214,7 @@ void SigInsertInternalCodes(Sig_p sig)
    SigSetPolymorphic(sig, sig->eqn_code, true);
 
    sig->neqn_code   = SigInsertId(sig, "$neq",   2, true);
-   SigSetPolymorphic(sig, sig->eqn_code, true);
+   SigSetPolymorphic(sig, sig->neqn_code, true);
 
    sig->qex_code   = SigInsertId(sig, "$qex",   2, true);
    sig->qall_code  = SigInsertId(sig, "$qall",  2, true);
@@ -262,11 +263,13 @@ void SigInsertInternalCodes(Sig_p sig)
    f_code =
 #endif
    SigInsertId(sig, "$ite", 3, true);
+   // SigSetPolymorphic(sig, SIG_ITE_CODE, true);
    assert(f_code == SIG_ITE_CODE); //for future code changes
 #ifndef NDEBUG
    f_code =
 #endif
    SigInsertId(sig, "$let", 3, true);
+   // SigSetPolymorphic(sig, SIG_LET_CODE, true);
    assert(f_code == SIG_LET_CODE); //for future code changes
 
    Type_p* args = TypeArgArrayAlloc(2);
@@ -620,19 +623,40 @@ void SigSetAllSpecial(Sig_p sig, bool value)
 //
 /----------------------------------------------------------------------*/
 
+#define MULTI_ARITY_HACK
+
 FunCode SigInsertId(Sig_p sig, const char* name, int arity, bool special_id)
 {
    long      pos;
    StrTree_p new, test;
+   DStr_p    fix_name = NULL;
 
    pos = SigFindFCode(sig, name);
 
    if(pos) /* name is already known */
    {
-      if(sig->f_info[pos].arity != arity)
+      if(sig->f_info[pos].arity != arity && problemType == PROBLEM_FO)
       {
-         printf("Problem: %s %d != %d\n", name, arity, sig->f_info[pos].arity);
+         //printf("Problem: %s %d != %d\n", name, arity, sig->f_info[pos].arity);
+#ifdef MULTI_ARITY_HACK
+         fix_name = DStrAlloc();
+         DStrAppendStr(fix_name, name);
+         DStrAppendStr(fix_name, "_ARITYFIX");
+         DStrAppendInt(fix_name, arity);
+         DStrAppendStr(fix_name, " ");  /* Trailing space should ensure that it
+                               * cannot come from the real parser */
+         name = DStrView(fix_name);
+         pos = SigFindFCode(sig, name);
+#else
          return 0; /* ...but incompatible */
+#endif
+      }
+   }
+   if(pos)
+   {
+      if(fix_name)
+      {
+         DStrFree(fix_name);
       }
       if(special_id)
       {
@@ -666,9 +690,73 @@ FunCode SigInsertId(Sig_p sig, const char* name, int arity, bool special_id)
    SigSetSpecial(sig,sig->f_count,special_id);
    sig->alpha_ranks_valid = false;
 
+   if(fix_name)
+   {
+      DStrFree(fix_name);
+   }
    return sig->f_count;
 }
 
+
+/*-----------------------------------------------------------------------
+//
+// Function: SigPopId()
+//
+//   Remove the last symbol from the signature. This should only be
+//   done when no structures use it - otherwise the behaviour is
+//   undefined. This also ignores the Let-Id-Stack. The function
+//   returns the old sig->f_count (equivalent to the removed
+//   identifier), or 0 if the signature is empty.
+//
+// Global Variables: -
+//
+// Side Effects    : Memory operations
+//
+/----------------------------------------------------------------------*/
+
+FunCode SigPopId(Sig_p sig)
+{
+   FunCode res = 0;
+
+   if(sig->f_count)
+   {
+      res = sig->f_count;
+      StrTreeDeleteEntry(&(sig->f_index), sig->f_info[sig->f_count].name);
+      // Identifier is freed (unexpectedly?) in StrTreeDeleteEntry()
+      //FREE(sig->f_info[sig->f_count].name);
+      sig->f_count--;
+   }
+   return res;
+}
+
+
+/*-----------------------------------------------------------------------
+//
+// Function: SigBacktrack()
+//
+//    Remove all symbols with f_codes > f_count from the signature. See
+//    SigPopId() for caveats. Returns the number of symbols popped.
+//
+// Global Variables: -
+//
+// Side Effects    : -
+//
+/----------------------------------------------------------------------*/
+
+long SigBacktrack(Sig_p sig, FunCode f_count)
+{
+   long res = 0;
+
+   while(sig->f_count > f_count)
+   {
+      res++;
+      SigPopId(sig);
+   }
+   //printf("Signature backtracked to %ld\n", f_count);
+   return res;
+}
+
+
 /*-----------------------------------------------------------------------
 //
 // Function: SigInsertLetId()
@@ -1483,8 +1571,11 @@ void SigDeclareType(Sig_p sig, FunCode f, Type_p type)
 
 void SigDeclareFinalType(Sig_p sig, FunCode f_code, Type_p type)
 {
-   SigDeclareType(sig, f_code, type);
-   SigFixType(sig, f_code);
+   if(!SigIsPolymorphic(sig, f_code))
+   {
+      SigDeclareType(sig, f_code, type);
+      SigFixType(sig, f_code);
+   }
 }
 
 
@@ -1527,7 +1618,6 @@ void SigDeclareIsFunction(Sig_p sig, FunCode f_code)
    }
 }
 
-
 /*-----------------------------------------------------------------------
 //
 // Function: SigDeclareIsPredicate()
@@ -1563,7 +1653,6 @@ void SigDeclareIsPredicate(Sig_p sig, FunCode f_code)
    }
 }
 
-
 /*-----------------------------------------------------------------------
 //
 // Function: SigPrintTypes()
@@ -1603,13 +1692,11 @@ void SigPrintTypes(FILE* out, Sig_p sig)
    }
 }
 
-
 /*-----------------------------------------------------------------------
 //
 // Function: SigPrintTypeDeclsTSTP()
 //
-//   Print TPTP-3 type declarations for all real symbols.
-//
+//   Print TPTP-3 type declarations for all real symbols in sig.
 //
 // Global Variables: -
 //
@@ -1625,8 +1712,8 @@ void SigPrintTypeDeclsTSTP(FILE* out, Sig_p sig)
 
    for(i=sig->internal_symbols+1; i <= sig->f_count; i++)
    {
-      fun = &sig->f_info[i];
-      if (fun->type)
+      fun = &(sig->f_info[i]);
+      if (fun->type /*&& !TypeIsUntyped(fun->type)*/)
       {
          fprintf(out, "%s(decl_%ld, type, %s: ", tag, i, fun->name);
          TypePrintTSTP(out, sig->type_bank, fun->type);
@@ -1636,6 +1723,39 @@ void SigPrintTypeDeclsTSTP(FILE* out, Sig_p sig)
 }
 
 
+/*-----------------------------------------------------------------------
+//
+// Function: SigPrintTypeDeclsTSTPSelective()
+//
+//   Print TPTP-3 type declarations for the symbols in sig that are
+//   also in symbols.
+//
+// Global Variables: -
+//
+// Side Effects    : IO on the file descriptor
+//
+/----------------------------------------------------------------------*/
+
+void SigPrintTypeDeclsTSTPSelective(FILE* out, Sig_p sig, NumTree_p *symbols)
+{
+   FunCode i;
+   Func_p fun;
+   const char* tag = problemType == PROBLEM_HO ? "thf" : "tff";
+
+   for(i=sig->internal_symbols+1; i <= sig->f_count; i++)
+   {
+      if(NumTreeFind(symbols,i))
+      {
+         fun = &sig->f_info[i];
+         if (fun->type /*&& !TypeIsUntyped(fun->type)*/)
+         {
+            fprintf(out, "%s(decl_%ld, type, %s: ", tag, i, fun->name);
+            TypePrintTSTP(out, sig->type_bank, fun->type);
+            fprintf(out, ").\n");
+         }
+      }
+   }
+}
 
 
 /*-----------------------------------------------------------------------
@@ -1643,7 +1763,7 @@ void SigPrintTypeDeclsTSTP(FILE* out, Sig_p sig)
 // Function: SigParseTFFTypeDeclaration()
 //
 //    Parses a type declaration, and update the signature if it is a
-//    symbol declaration (ignores type declarations)
+//    symbol declaration.
 //
 // Global Variables: -
 //
@@ -1671,7 +1791,7 @@ void SigParseTFFTypeDeclaration(Scanner_p in, Sig_p sig)
    symbtype = FuncSymbParse(in, id);
    if(symbtype == FSIdentVar || symbtype == FSNone)
    {
-      Error("expected symbol in type declaration", OTHER_ERROR);
+      Error("expected type name in type declaration", OTHER_ERROR);
    }
 
    /* parse type */
@@ -1683,7 +1803,6 @@ void SigParseTFFTypeDeclaration(Scanner_p in, Sig_p sig)
       AcceptInpTok(in, CloseBracket);
    }
 
-   /* we only keep declarations of symbols, not declaration of types */
    if(!TypeIsTypeConstructor(type))
    {
       int arity = TypeIsArrow(type) ? type->arity - 1 : 0;
@@ -1734,6 +1853,61 @@ bool SigHasUnimplementedInterpretedSymbols(Sig_p sig)
    return false;
 }
 
+
+
+/*-----------------------------------------------------------------------
+//
+// Function: SigFCodesCollectTypes()
+//
+//   Collect all types of symbols in fcodes (and their proper subtypes
+//   in the first-order sense) into types.
+//
+// Global Variables: -
+//
+// Side Effects    : -
+//
+/----------------------------------------------------------------------*/
+
+long SigFCodesCollectTypes(Sig_p sig, NumTree_p fcodes, PTree_p *types)
+{
+   long res = 0;
+   PStack_p to_process = PStackAlloc(), iter;
+   NumTree_p handle;
+   Type_p type;
+
+   iter = NumTreeTraverseInit(fcodes);
+   while((handle = NumTreeTraverseNext(iter)))
+   {
+      type = SigGetType(sig, handle->key);
+      //printf("# Symbol %ld = %s has type %p\n", handle->key,
+      //SigFindName(sig, handle->key), type);
+      if(type)
+      {
+         PStackPushP(to_process, type);
+      }
+   }
+   NumTreeTraverseExit(iter);
+
+   while(!PStackEmpty(to_process))
+   {
+      type = PStackPopP(to_process);
+      if(!PTreeFind(types, type))
+      {
+         res++;
+         PTreeStore(types, type);
+         for(int i=0; i<type->arity; i++)
+         {
+            PStackPushP(to_process, type->args[i]);
+         }
+      }
+   }
+   PTreeTraverseExit(to_process);
+   return res;
+}
+
+
+
+
 /*-----------------------------------------------------------------------
 //
 // Function: SigUpdateFeatureOffset()
@@ -1881,6 +2055,7 @@ bool SigSymbolUnifiesWithVar(Sig_p sig, FunCode f_code)
 
    return problemType == PROBLEM_HO ||
           f_code == SIG_TRUE_CODE || f_code == SIG_FALSE_CODE ||
+          f_code != SIG_DB_LAMBDA_CODE ||
           f_code <= 0 ||
           !SigIsPredicate(sig,f_code);
 }
@@ -1944,7 +2119,7 @@ void SigEnterLetScope(Sig_p sig, PStack_p type_decls)
 //
 /----------------------------------------------------------------------*/
 
-void  SigExitLetScope(Sig_p sig)
+void SigExitLetScope(Sig_p sig)
 {
    PStack_p scope = PStackPopP(sig->let_scopes);
 
@@ -1964,10 +2139,33 @@ void  SigExitLetScope(Sig_p sig)
       }
 
    }
-
    PStackFree(scope);
 }
 
+/*-----------------------------------------------------------------------
+//
+// Function: SigHasChoiceSym()
+//
+//   Count number of symbols with a given arity. If predictates is
+//   true, count predicates, otherwise count function symbols.
+//
+// Global Variables: -
+//
+// Side Effects    : -
+//
+/----------------------------------------------------------------------*/
+
+bool SigHasChoiceSym(Sig_p sig)
+{
+   FunCode i;
+
+   for(i=sig->internal_symbols+1;
+       i<=sig->f_count && !IsChoiceType(SigGetType(sig, i));
+       i++)
+   {}
+   return i<=sig->f_count;
+}
+
 
 /*---------------------------------------------------------------------*/
 /*                        End of File                                  */
